@@ -1,5 +1,5 @@
 #
-# $Id: Rest.pm,v e5e3576f73a3 2015/11/10 09:17:44 gomor $
+# $Id: Rest.pm,v 629ac2fb6bcb 2016/02/13 17:10:24 gomor $
 #
 # client::rest Brik
 #
@@ -11,9 +11,15 @@ use base qw(Metabrik::Client::Www);
 
 sub brik_properties {
    return {
-      revision => '$Revision: e5e3576f73a3 $',
-      tags => [ qw(unstable http client rest api) ],
+      revision => '$Revision: 629ac2fb6bcb $',
+      tags => [ qw(unstable http api) ],
+      author => 'GomoR <GomoR[at]metabrik.org>',
+      license => 'http://opensource.org/licenses/BSD-3-Clause',
       attributes => {
+         uri => [ qw(uri) ],  # Inherited
+         username => [ qw(username) ],  # Inherited
+         password => [ qw(password) ],  # Inherited
+         ssl_verify => [ qw(0|1) ], # Inherited
          output_mode => [ qw(json|xml) ],
       },
       attributes_default => {
@@ -32,7 +38,6 @@ sub brik_properties {
          content => [ ],
       },
       require_modules => {
-         'Metabrik::String::Uri' => [ ],
          'Metabrik::String::Xml' => [ ],
          'Metabrik::String::Json' => [ ],
       },
@@ -50,10 +55,10 @@ sub content {
    my $sm;
    my $output_mode = $self->output_mode;
    if ($output_mode eq 'json') {
-      $sm = Metabrik::String::Json->new_from_brik($self) or return;
+      $sm = Metabrik::String::Json->new_from_brik_init($self) or return;
    }
    elsif ($output_mode eq 'xml') {
-      $sm = Metabrik::String::Xml->new_from_brik($self) or return;
+      $sm = Metabrik::String::Xml->new_from_brik_init($self) or return;
    }
    else {
       return $self->log->error("content: output_mode not supported [$output_mode]");
@@ -72,7 +77,7 @@ Metabrik::Client::Rest - client::rest Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2015, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
