@@ -1,5 +1,5 @@
 #
-# $Id: Www.pm,v b0ac8f9260b0 2016/02/18 17:50:27 gomor $
+# $Id: Www.pm,v ae4d13e0fd61 2016/04/02 12:37:00 gomor $
 #
 # client::www Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::System::Package);
 
 sub brik_properties {
    return {
-      revision => '$Revision: b0ac8f9260b0 $',
+      revision => '$Revision: ae4d13e0fd61 $',
       tags => [ qw(unstable browser http javascript screenshot) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -25,6 +25,7 @@ sub brik_properties {
          datadir => [ qw(datadir) ],
          timeout => [ qw(0|1) ],
          rtimeout => [ qw(timeout) ],
+         add_headers => [ qw(http_headers_hash) ],
          _client => [ qw(object|INTERNAL) ],
          _last => [ qw(object|INTERNAL) ],
       },
@@ -33,6 +34,7 @@ sub brik_properties {
          ignore_content => 0,
          timeout => 0,
          rtimeout => 10,
+         add_headers => {},
       },
       commands => {
          install => [ ], # Inherited
@@ -164,6 +166,8 @@ sub _method {
       $self->_client($client);
    }
 
+   my $add_headers = $self->add_headers;
+
    $self->log->verbose("$method: $uri");
 
    my $response;
@@ -172,7 +176,7 @@ sub _method {
          $response = $client->$method($uri, Content => $data);
       }
       elsif ($method eq 'options' || $method eq 'patch') {
-         my $req = HTTP::Request->new($method => $uri);
+         my $req = HTTP::Request->new($method, $uri, $add_headers);
          $response = $client->request($req);
       }
       else {
