@@ -1,5 +1,5 @@
 #
-# $Id: Package.pm,v b86015aa8479 2016/01/15 17:13:51 gomor $
+# $Id: Package.pm,v 7d4af34f011c 2016/04/12 05:55:37 gomor $
 #
 # system::freebsd::package Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command);
 
 sub brik_properties {
    return {
-      revision => '$Revision: b86015aa8479 $',
+      revision => '$Revision: 7d4af34f011c $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -52,10 +52,17 @@ sub install {
    my $ref = $self->brik_help_run_invalid_arg('install', $package, 'ARRAY', 'SCALAR')
       or return;
 
-   my $cmd = 'pkg install ';
-   $ref eq 'ARRAY' ? ($cmd .= join(' ', @$package)) : ($cmd .= $package);
+   my $r;
+   if ($ref eq 'ARRAY') {
+      for my $this (@$package) {
+         $r = $self->sudo_system('pkg install '.$this);
+      }
+   }
+   else {
+      $r = $self->sudo_system('pkg install '.$package);
+   }
 
-   return $self->sudo_system($cmd);
+   return $r;
 }
 
 sub update {
