@@ -1,5 +1,5 @@
 #
-# $Id: Os.pm,v 1cdea52670de 2016/01/02 17:14:57 gomor $
+# $Id: Os.pm,v 2dab3f76f3fa 2016/01/18 10:59:08 gomor $
 #
 # system::os brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1cdea52670de $',
+      revision => '$Revision: 2dab3f76f3fa $',
       tags => [ qw(unstable uname linux freebsd distribution) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -135,6 +135,31 @@ sub distribution {
             description => $info{DISTRIB_DESCRIPTION}, # Ubuntu 14.10
          };
       }
+      elsif (-f '/etc/redhat-release') {
+         return {
+            name => 'RedHat',  # RedHat
+         };
+      }
+      elsif (-f '/etc/os-release') {
+         return {
+            name => 'OpenSUSE',  # OpenSUSE
+         };
+      }
+      elsif (-f '/etc/arch-release') {
+         return {
+            name => 'ArchLinux',  # ArchLinux
+         };
+      }
+      elsif (-f '/etc/manjaro-release') {
+         return {
+            name => 'Manjaro Linux',  # Manjaro Linux
+         };
+      }
+      elsif (-f '/etc/gentoo-release') {
+         return {
+            name => 'Gentoo',  # Gentoo
+         };
+      }
    }
 
    # Default
