@@ -1,5 +1,5 @@
 #
-# $Id: Process.pm,v 714d5f4c8eff 2015/11/22 11:38:52 gomor $
+# $Id: Process.pm,v 9c71f585c5e3 2016/02/08 20:37:51 gomor $
 #
 # system::process Brik
 #
@@ -11,31 +11,41 @@ use base qw(Metabrik::Shell::Command);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 714d5f4c8eff $',
-      tags => [ qw(unstable system process) ],
+      revision => '$Revision: 9c71f585c5e3 $',
+      tags => [ qw(unstable) ],
+      author => 'GomoR <GomoR[at]metabrik.org>',
+      license => 'http://opensource.org/licenses/BSD-3-Clause',
       attributes => {
          datadir => [ qw(datadir) ],
          force_kill => [ qw(0|1) ],
-         close_output_on_daemonize => [ qw(0|1) ],
+         close_output_on_start => [ qw(0|1) ],
       },
       attributes_default => {
          force_kill => 0,
-         close_output_on_daemonize => 1,
+         close_output_on_start => 1,
       },
       commands => {
+         parse_ps_output => [ qw(data) ],
          list => [ ],
          is_running => [ qw(process) ],
          get_process_info => [ qw(process) ],
          kill => [ qw(process|pid) ],
-         daemonize => [ qw($sub) ],
+         start => [ qw($sub) ],
          list_daemons => [ ],
          get_latest_daemon_id => [ ],
          kill_from_pidfile => [ qw(pidfile) ],
          is_running_from_pidfile => [ qw(pidfile) ],
+         grep_by_name => [ qw(process_name) ],
+         get_new_pidfile => [ ],
+         get_latest_pidfile => [ ],
+         write_pidfile => [ ],
+         delete_pidfile => [ qw(pidfile) ],
+         wait_for_pidfile => [ qw(pidfile) ],
       },
       require_modules => {
          'Daemon::Daemonize' => [ ],
          'POSIX' => [ qw(:sys_wait_h) ],
+         'Time::HiRes' => [ qw(usleep) ],
          'Metabrik::File::Find' => [ ],
       },
       require_binaries => {
@@ -44,16 +54,17 @@ sub brik_properties {
    };
 }
 
-sub list {
+sub parse_ps_output {
    my $self = shift;
+   my ($data) = @_;
 
-   my $res = $self->capture("ps awuxw") or return;
+   $self->brik_help_run_undef_arg('parse_ps_output', $data) or return;
 
    my @a = ();
-   my $first = shift @$res;
+   my $first = shift @$data;
    my $header = [ split(/\s+/, $first) ];
    my $count = scalar(@$header) - 1;
-   for my $this (@$res) {
+   for my $this (@$data) {
       my @toks = split(/\s+/, $this);
       my $h = {};
       for my $n (0..$count) {
@@ -71,13 +82,19 @@ sub list {
    return \@a;
 }
 
+sub list {
+   my $self = shift;
+
+   my $res = $self->capture("ps awuxw") or return;
+
+   return $self->parse_ps_output($res);
+}
+
 sub is_running {
    my $self = shift;
    my ($process) = @_;
 
-   if (! defined($process)) {
-      return $self->log->error($self->brik_help_run('is_running'));
-   }
+   $self->brik_help_run_undef_arg('is_running', $process) or return;
 
    my $list = $self->list or return;
    for my $this (@$list) {
@@ -96,14 +113,12 @@ sub get_process_info {
    my $self = shift;
    my ($process) = @_;
 
-   if (! defined($process)) {
-      return $self->log->error($self->brik_help_run('is_running'));
-   }
+   $self->brik_help_run_undef_arg('get_process_info', $process) or return;
 
    my @results = ();
    my $list = $self->list or return;
    for my $this (@$list) {
-      my $command = $this->{COMMAND};
+      my $command = $this->{COMMAND} or next;
       my @toks = split(/\s+/, $command);
       $toks[0] =~ s/^.*\/(.*?)$/$1/;
       if ($toks[0] eq $process) {
@@ -118,9 +133,7 @@ sub kill {
    my $self = shift;
    my ($process) = @_;
 
-   if (! defined($process)) {
-      return $self->log->error($self->brik_help_run('kill'));
-   }
+   $self->brik_help_run_undef_arg('kill', $process) or return;
 
    my $signal = $self->force_kill ? 'KILL' : 'TERM';
 
@@ -139,60 +152,42 @@ sub kill {
    return 1;
 }
 
-sub daemonize {
+#
+# start process sub from user program has to call write_pidfile Command to create the pidfile.
+# Then parent process can use wait_for_pidfile Command with the $pidfile returned from 
+# start Command to wait for the process to really start.
+#
+sub start {
    my $self = shift;
    my ($sub) = @_;
 
    my %opts = (
-      close => $self->close_output_on_daemonize,
+      close => $self->close_output_on_start,
    );
 
-   my $new_pid;
-   my $pidfile;
+   my $r;
    # Daemonize the given subroutine
    if (defined($sub)) {
-      my $id = $self->get_latest_daemon_id;
-      defined($id) ? $id++ : ($id = 1);
-      $pidfile = $self->datadir."/daemonpid.$id";
-
-      my $r = Daemon::Daemonize->daemonize(
+      $r = Daemon::Daemonize->daemonize(
          %opts,
          run => $sub,
       );
-
-      # Waiting for new pidfile to be created, but no more than 100_000 loops.
-      #my $count = 100_000;
-      #while (! ($new_pid = Daemon::Daemonize->read_pidfile($pidfile))) {
-         #last if ++$count == 100_00;
+      #if (! defined($r)) {
+         #return $self->log->error("start: failed from sub");
       #}
-      my $written = 0;
-      my $count = 100_000;
-      my $sp = Metabrik::System::Process->new_from_brik_init($self) or return;
-      while (1) {
-         my $list = $sp->get_process_info("arpspoof") or return;
-         for (@$list) {
-            # First process if the good one
-            if (exists($_->{PID})) {
-               $new_pid = $_->{PID};
-               Daemon::Daemonize->write_pidfile($pidfile, $new_pid);
-               $written++;
-               last;
-            }
-         }
-         last if $written;
-         last if ++$count == 100_000;
-      }
    }
-   # Or myself. But no handling of pidfile there.
+   # Or myself.
    else {
-      Daemon::Daemonize->daemonize(
+      $r = Daemon::Daemonize->daemonize(
          %opts,
       );
    }
 
-   $self->log->verbose("daemonize: new daemon with pid [$new_pid] started");
+   my $pidfile = $self->get_new_pidfile or return;
 
-   return defined($new_pid) ? $pidfile : 1;
+   $self->log->verbose("start: new daemon started with pidfile [$pidfile]");
+
+   return $pidfile;
 }
 
 sub list_daemons {
@@ -232,9 +227,8 @@ sub kill_from_pidfile {
    my $self = shift;
    my ($pidfile) = @_;
 
-   if (! defined($pidfile)) {
-      return $self->log->error($self->brik_help_run('kill_from_pidfile'));
-   }
+   $self->brik_help_run_undef_arg('kill_from_pidfile', $pidfile) or return;
+   $self->brik_help_run_file_not_found('kill_from_pidfile', $pidfile) or return;
 
    if (my $pid = Daemon::Daemonize->check_pidfile($pidfile)) {
       $self->log->verbose("kill_from_pidfile: file[$pidfile] and pid[$pid]");
@@ -249,17 +243,128 @@ sub is_running_from_pidfile {
    my $self = shift;
    my ($pidfile) = @_;
 
-   if (! defined($pidfile)) {
-      return $self->log->error($self->brik_help_run('is_running_from_pidfile'));
+   $self->brik_help_run_undef_arg('is_running_from_pidfile', $pidfile) or return;
+
+   # Not file found, so probably not running
+   if (! -f $pidfile) {
+      return 0;
    }
 
    if (my $pid = Daemon::Daemonize->check_pidfile($pidfile)) {
+      $self->debug && $self->log->debug("is_running_from_pidfile: yes");
       return 1;
    }
 
+   $self->debug && $self->log->debug("is_running_from_pidfile: no");
+
    return 0;
 }
 
+sub grep_by_name {
+   my $self = shift;
+   my ($process_name) = @_;
+
+   $self->brik_help_run_undef_arg('grep_by_name', $process_name) or return;
+
+   my $list = $self->list or return;
+   for my $p (@$list) {
+      if (lc($p->{COMMAND}) =~ m{$process_name}i) {
+         return $p;
+      }
+   }
+
+   return 0;
+}
+
+sub get_new_pidfile {
+   my $self = shift;
+
+   my $id = $self->get_latest_daemon_id;
+   defined($id) ? $id++ : ($id = 1);
+   my $pidfile = sprintf("%s/daemonpid.%05d", $self->datadir, $id);
+
+   return $pidfile;
+}
+
+sub get_latest_pidfile {
+   my $self = shift;
+
+   my $pidfile;
+   my $id = $self->get_latest_daemon_id;
+   if (defined($id)) {
+      $pidfile = sprintf("%s/daemonpid.%05d", $self->datadir, $id);
+   }
+   else {
+      return $self->log->error("get_latest_pidfile: no pidfile found");
+   }
+
+   return $pidfile;
+}
+
+#
+# To be called by a sub used by start Command
+#
+sub write_pidfile {
+   my $self = shift;
+
+   my $pidfile = $self->get_new_pidfile or return;
+
+   my $pid = $$;
+
+   my $r = Daemon::Daemonize->write_pidfile($pidfile, $pid);
+   if (! defined($r)) {
+      return $self->log->erro("write_pidfile: failed to write pidfile [$pidfile]: $!");
+   }
+
+   return $pid;
+}
+
+#
+# To be used by parent process
+#
+sub delete_pidfile {
+   my $self = shift;
+   my ($pidfile) = @_;
+
+   $self->brik_help_run_undef_arg('delete_pidfile', $pidfile) or return;
+
+   if (! -f $pidfile) {
+      # Nothing to delete
+      return 0;
+   }
+
+   my $r = Daemon::Daemonize->delete_pidfile($pidfile);
+   if (! defined($r)) {
+      return $self->log->erro("delete_pidfile: failed to delete pidfile [$pidfile]: $!");
+   }
+
+   return 1;
+}
+
+# XXX: Move to system::file and use a helper here
+#
+# To be used by parent process
+#
+sub wait_for_pidfile {
+   my $self = shift;
+   my ($pidfile) = @_;
+
+   $self->brik_help_run_undef_arg('wait_pidfile', $pidfile) or return;
+
+   my $found = 0;
+   # 50 * 100 ms = 5s
+   for (0..49) {
+      if (-e $pidfile) {
+         $found++;
+         last;
+      }
+      Time::HiRes::usleep(100_000); # 100_000us => 100ms => 0.1s
+      # 0.1s * 50 = 5s
+   }
+
+   return $found;
+}
+
 1;
 
 __END__
@@ -270,7 +375,7 @@ Metabrik::System::Process - system::process Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2015, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
